# Automatics - Valheim Mod
Automatics is a mod that automates the tedious tasks of life in Valheim.
Most of its features exist in existing mods, but it has been re-designed
to make it easier for me to use.

# Features

<div class="tip">

Each of the features described below can be completely disabled using
the "Disable Module" option in the configuration.

</div>

## Automatic door

Automatically opens and closes the door near the player. The interval
and distance to detect the player can be changed from the configuration.

## Automatic mapping

Automatic pinning animals, monsters, floras, veins, dungeons, etc. that
exist around the player to the map. The pinning allows for each object
and the detection range of the object can be changed from the config.

<div class="informalexample">

**Custom icon pack:**
You can also define your own icons in png and json files. See
[docs/custom-icon-pack.adoc](https://github.com/eideehi/valheim-automatics/blob/1.4.3/docs/custom-icon-pack.adoc)
for custom icon pack specifications.

[![Document - Custom Icon Pack](https://app.box.com/shared/static/ggj61oyrdik1jk08lohdqr91e1q5isqv.png)](https://github.com/eideehi/valheim-automatics/blob/1.4.3/docs/custom-icon-pack.adoc)

</div>

## Automatic processing

Refueling pieces that need fuel. Deliver materials to pieces that
process materials, and store items produced by pieces. These tasks can
be automated via containers around the piece.

## Automatic feeding

Modifies tamable creatures so that they can eat food from containers and
the player’s inventory, not just items on the ground.

## Automatic repair

It can automatically repair items when the player is near a crafting
station or when the crafting station GUI is opened or automatically
repair nearby pieces when the player has a hammer equipped.

## Automatic mining

Automatically mines minerals in the player’s surroundings. You can also
add a shortcut key that allows you to mine at any time.

## Automatic pickup

Automatically pick up items around the player. You can also add a
shortcut key that searches the surroundings for the same items as the
interacted item and pick up them all.

# Console commands

Automatics add a few commands to help the user.

## automatics
*Usage: automatics (OPTIONS…​)*

Displays the usage of commands added by Automatics.

OPTIONS:
| Option              | Description                                                               |
|---------------------|---------------------------------------------------------------------------|
| -i, --include=VALUE | Show only commands whose names include the word specified in this option. |
| -e, --exclude=VALUE | Exclude commands whose names contain the word specified with this option. |
| -v, --verbose       | Displays detailed usage of the command.                                   |
| -h, --help          | Displays a help message and exits the command.                            |

## printnames
*Usage: printnames (OPTIONS…​) (WORD\|REGEXP)…​*

Outputs internal or display names that contain the specified string or
match the specified regular expression. If multiple arguments are
specified, only those matching all of them will be output.

WORD
A text contained in the internal or display name. (e.g. `$enemy_`,
`$item_`, `$piece_`, `Boar`, `Mushroom`, `Wood door`); All partially matching
internal and display names are output.

REGEXP
A regular expression of the internal or display name to be output. Must
be prefixed with r/ (e.g. `r/^[$]item\_`, `r/^boar$`)

OPTIONS:
| Option     | Description                                    |
|------------|------------------------------------------------|
| -h, --help | Displays a help message and exits the command. |

Examples:
-   `printnames ling r/^[$]enemy_`

-   `printnames r/^[$@]location_.+(?<!_(enter|exit))$`

-   `printnames mushroom r/^[$]item_.+(?<!_description)$`

## printobjects
*Usage: printobjects \[TYPE\] (OPTIONS…​)*

Display objects that can be handled by Automatics in the nearby.

TYPE
Type of object to be displayed. Specify one of the following: animal,
monster, flora, mineral, spawner, vehicle, dungeon, spot, other.

OPTIONS:
| Option              | Description                                                                                                                                                                           |
|---------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| -r, --radius=VALUE  | Specify the range within which the object is to be searched. \[Default: 32\] (Unit: Meters)                                                                                           |
| -n, --number=VALUE  | Specify how many objects matching the condition are to be displayed. \[Default: 4\]                                                                                                   |
| -i, --include=VALUE | Show only objects whose internal or display names match the word specified in this option. It works as a regular expression by concatenating r/ at the beginning of the string.       |
| -e, --exclude=VALUE | Exclude objects whose internal names or display names match the word specified with this option. It works as a regular expression by concatenating r/ at the beginning of the string. |
| -h, --help          | Displays a help message and exits the command.                                                                                                                                        |

<div class="note">

The result of this command may show objects that you feel are not of the
target type. For example, a Flint appear in the Flora search. This is
not a bug but means that Automatics can treat Flint as Flora.

</div>

# Configurations

I recommend using [Configuration
Manager](https://github.com/BepInEx/BepInEx.ConfigurationManager).

[![Configuration Menu (Click to view full size)](https://app.box.com/shared/static/3v57rjpauzzyv0xeugohnw8bn2ye3q2h.png)](https://app.box.com/shared/static/vfzsn69i950l48er2u69tssod6xxsh8u.jpg)

**The README would be too large if we described all the details of the
configuration, so we split it into separate file.**

Open
[CONFIG.adoc](https://github.com/eideehi/valheim-automatics/blob/1.4.3/CONFIG.adoc)
to see the configuration details.

## Adding object definitions to Automatics

You can use the [Configuration
Manager](https://github.com/BepInEx/BepInEx.ConfigurationManager) to
define objects that you want Automatics to work with.

![User-defined objects in GUI](https://app.box.com/shared/static/5f6dvpg1elczu9froqkepxamv03ci9cd.png)

Open
[docs/add-user-defined-object.adoc](https://github.com/eideehi/valheim-automatics/blob/1.4.3/docs/add-user-defined-object.adoc)
to learn more about adding user-defined objects.

## About deprecated options

### Resources Directory / \[resources_directory\]
Provided to load custom icon packs, this option will be discontinued in
the near future as Automatics can now load custom icon packs from the
BepInEx plugin folder.

# Supplementary explanation

## Matching by "Display name" and "Internal name"

In some features of Automatics, there is an option that allows the user
to add targets as needed. The "Display name" and "Internal name" are
used to identify these targets. The display name and internal name are
matched according to different rules.

### Display name
Display names are the names that appear in the game, such as Boar, Deer,
Dandelion, etc. The matching rule for "Display name" is a partial match,
meaning that if the target display name contains the specified string,
it matches. It is case-insensitive.

### Internal name
Internal names are the names used inside the game program, such as
`$enemy_boar`, `$enemy_deer`, `$item_dandelion`, etc. The matching rule
for "Internal name" is an exact match, meaning that if the target
internal name is identical to the specified string, it matches. It is
case-insensitive. Note that internal names for translations added by
Automatics are prefixed with `@`, not `$`, as in `@internal_name`

### Matching Samples

| Display name | Internal name    |
|--------------|------------------|
| Greyling     | $enemy_greyling  |
| Greydwarf    | $enemy_greydwarf |
| Surtling     | $enemy_surtling  |

Target data

|           | Grey     | ling     | $enemy_greyling | $enemy_greydwarf | $enemy\_ |
|-----------|----------|----------|-----------------|------------------|----------|
| Greyling  | Match    | Match    | Match           | No match         | No match |
| Greydwarf | Match    | No match | No match        | Match            | No match |
| Surtling  | No match | Match    | No match        | No match         | No match |

Matching result

# Languages

| Language | Translators       | Status |
|----------|-------------------|--------|
| English  | Translation Tools | 100%   |
| Japanese | EideeHi           | 100%   |

# Contacts

[![Bug report on Issues](https://app.box.com/shared/static/g2v3vbju4jazq7kycoigp60ltki2kw8i.png)](https://github.com/eideehi/valheim-better-portal/issues)

**Only bug reports are accepted under Issues.**

[![eidee.net - Discord Server](https://app.box.com/shared/static/0s09ti60hvyyp5k98xyrnkfp683mrt9r.png)](https://discord.gg/DDQqxkK7s6)

**Questions, suggestions and comments are welcome on the Discord Server.**

# Credits

-   Dependencies:

    -   [Configuration
        Manager](https://github.com/BepInEx/BepInEx.ConfigurationManager)

    -   [LitJSON](https://litjson.net)

    -   [NDesk.Options](http://ndesk.org/Options)

# License

Automatics is developed and released under the MIT license. For the full
text of the license, please see the
[LICENSE](https://github.com/eideehi/valheim-automatics/blob/1.4.3/LICENSE)
file.

## Changelog
#### v1.4.3 [2023-03-18]
- Updated for changes in Valheim 0.214.2.
- Fixed a bug where the "Pickup All Nearby" shortcut key wouldn't work if it included an interact key (e.g. Shift + E).
- Fixed a bug that caused wild animals to attack containers with food under certain conditions, even if they were allowed to eat from them.
- Fixed a bug where portals wouldn't be automatically pinned, even if "Allow Pinning Portal" was enabled.
- Fixed a bug where pins would remain on the map even after disabling automatic mapping in the configuration.

#### v1.4.2 [2023-03-12]
- Fixed a bug where the game won't start if some modules are disabled.

#### v1.4.1 [2023-03-11]
- Fixed a bug where the toggle key for automatic doors would become disabled after using it to turn off the function.
- Fixed a bug where the automatic mapping function would delete unrelated pins, such as pins added by users or boss pins.
- Fixed a bug where the automatic mapping function would cause an exception when loading the world without exiting the game after logging out of the world.
- Fixed a bug where the automatic door closing function would not work, causing the door to remain open.
- The "printobjects" command can now output doors and containers.
- Fixed a bug where the automatic repair function would cause an exception under certain conditions.
- Fixed a bug where the "Distance For Automatic Opening" configuration for automatic doors was not functioning and instead referenced "Distance For Automatic Closing."
- Modified the "Disable Module" item in the configuration to be more intuitive.
- Added "Hexagonal gate" to the door presets.

#### v1.4.0 [2022-12-26]
- Overall Rewriting
  * AutomaticDoor
    - Logic Improvements
  * AutomaticFeeding
    - A little refactoring
  * AutomaticMapping
    - Add Vehicle category
    - Portal pinning is now a stand-alone option, not in the Other category
    - Static object pins are no longer saved by default. They can be changed to be saved in the options
    - You can now hold down the left Shift key and click on a pin on the map to force the pin to change to save
    - Added option to automatically remove pins when destructible objects are destroyed. (probably won't work perfectly, so don't expect too much)
  * AutomaticMining
    - Limit the number of parts that can be damaged by automatic mining to a maximum of 3 at a time
  * AutomaticProcessing
    - Removed the option to limit crafting by the total number of items in a product, instead added an option to limit crafting by the number of product stacks
    - Added option to replenish material only when the material in the processor runs out
    - Added option to refuel only when the fuel in the processor runs out
    - Smelter's automatic crafting has been improved to process time-based quantities of material even when away from it for extended periods of time
  * AutomaticRepair
    - A little refactoring
- Added AutomaticPickup feature
- Custom objects have been renamed User-defined objects and moved to the General category of the Config
  * I have documentation on how to add user-defined objects, so please read through it if necessary
  * https://github.com/eideehi/valheim-automatics/blob/1.4.0/docs/add-user-defined-object.adoc
- Mistlands content has been added to the automation
- Added options to completely disable each feature
- Change the console command to take arguments like the CLI

#### v1.3.2 [2022-12-06]
- Rebuild for Mistlands Update
  * Content added in the Mistlands Update is not yet available for automation by Automatics

#### v1.3.1 [2022-11-13]
- Fix an error that occurred if ConfigurationManager was not installed
- Custom map icons can now be treated as child mod of Automatics
  * For details, refer to the "Directory structure" section of the README
- Add option to hide pin names on custom map icon
  * For details, refer to the "options" entry in the "Format of custom-map-icon.json" section of the README
- Add option to set the scale at which custom map icons appear on the map
  * For details, refer to the "options" entry in the "Format of custom-map-icon.json" section of the README

#### v1.3.0 [2022-04-26]
- Add the feature of 'Automatic Mining'
- Add option to change the directory to load resources such as translation files and custom icons
- Add option to change the position of the message displayed when enabling/disabling the automatic door function with a shortcut key
- Change so that the slider for handling floating decimals in the configurator increases or decreases the numerical value by 0.01 increments
- Reviewed and corrected the key names and descriptions in the configuration options
  * The configurations are automatically migrated, but be sure to check the values of the configurations after starting the game to be sure
- Rewrote most of the automatic mapping feature
  * Fix inability to apply custom icons with internal names to birds
  * Change the deposit pinning location to the center of the object
    - Due to this change, large deposits such as copper may be pinned in duplicate with existing pins
  * Add custom options for other objects, dungeons, and spots

#### v1.2.1 [2022-04-19]
- Change the automatic door to not work if there is an obstacle between the player and the door
- Change the minimum automatic door process interval to 0
  * Setting the value below 0.1 disables automatic opening or closing. The substantive minimum interval remains unchanged from 0.1
- Change so that the printnames command searches all translations
  * With this change, the printnames command can now take multiple arguments, allowing for more powerful filtering
  * See [printnames usage](https://github.com/eideehi/valheim-automatics/blob/1.2.1/README.adoc#printnames-wordregexp) for command details

#### v1.2.0 [2022-04-16]
- Change the format of the Cutom Map Icon settings file from csv to json
  * See ["Format of custom-map-icon.json"](https://github.com/eideehi/valheim-automatics/tree/1.2.0/README.adoc#format-of-custom-map-iconjson) for more information on json files
- Change translation files format from csv to json
- Fix 'Standing brazier' was omitted from 'Automatic Processing'
- Fix to be able to output names of fish, birds and dungeons with the 'printnames' command
- Fix a bug that prevented some config values from being read accurately

#### v1.1.2 [2022-04-14]
- Add command to output internal and display names
  * See ["Console commands"](https://github.com/eideehi/valheim-automatics/tree/1.1.2/README.adoc#console-commands) in the README for details
- Add options to suppress automatic processing based on item count
- Add option to automatic refuel only when materials supplied
- Change 'Automatic Repair' not to work during game pauses
- Change 'Allow...(Custom)' options to evaluate for exact match if an internal name is specified, or partial match if a display name is specified
  * The "$enemy_boar" matches only "Boar", but "Greydwarf" matches "Greydwarf brute" and "Greydwarf shaman" in addition to "Greydwarf"
  * The same goes for TARGET for [Custom Map Icon](https://github.com/eideehi/valheim-automatics/tree/1.1.2/README.adoc#custom-map-icons)

#### v1.1.1 [2022-04-10]
- Fix package task of mod files for Thunderstore
- Fix a bug in which the 'Repair Pieces' function continued to work even though the 'Automatic Repair' feature was disabled in the configuration

#### v1.1.0 [2022-04-07]
- Add the feature of 'Automatic Feeding'
- Add the feature of 'Automatic Repair'
- Add option to allow automatic opening/closing for any door
- Fix a bug where the generated items would be lost if Smelter could not find the container when 'Automatic Store' was enabled
- Fix a bug where the honey to be set over the maximum if Beehive could not find the container when 'Automatic Store' was enabled

#### v1.0.6 [2022-04-04]
- Fix a bug that prevented automatic pinning of player existing in the open air under certain conditions, judging that they exist in the dungeon
- Add elements added in patch 0.207.20 to be the target for automatic pinning
- Change the maximum value of the container search range in 'Automatic Processing' to 64
- Change to remove pins added by automatic pinning when some veins are destroyed

#### v1.0.5 [2022-03-28]
- Add shortcut key to perform searches for static objects
- Add shortcut key to enable/disable automatic door
- Fix a bug that caused dynamic pins to remain on the map when auto-pinning was disabled from the config
- Change the initial value of the 'Automatic Map Pinning' configuration

#### v1.0.4 [2022-03-18]
- Fix a bug fish were not pinned
- Fix a bug that caused pins to be deleted at unintended times
- Fix a bug in which automatic pinning of ship was enabled regardless of the config value
- Add the feature to customize the icons of pins added by auto pinning
  * There are no custom icons bundled with Automatics, please add your own

#### v1.0.3 [2022-03-16]
- Fix a bug where tamed animals continue to be pinned
- Add options for users to add map pinning targets (animals, monsters, flora, veins, and spawners)
  * As a result, elements added by mods are no longer automatic pinning unless this option is used
- Add an option to allow ships to be automatic pinning
  * This is dynamic pinning, but unlike other dynamic pins, the pins is saved in the save data

#### v1.0.2 [2022-03-15]
- Improvement of FPS drop due to Automatic Map Pinning
  * In particular, FPS is greatly improved in areas with a lot of flora, such as farms
- Reduce the processing load on Automatic Door
- Add Portal to the Automatic Map Pinning target
  * Portal is added to 'Allow Pinning Other Objects'

#### v1.0.1 [2022-03-06]
- Fix a bug that prevented some Tar Pits from being pinning
- Fix a bug in which automatic map pinning of static objects causes a significant drop in FPS
- Fix logic for determining if a vein is in ground
- Change the Automatic Map Pinning process is not called when the game is paused
- Change Automatic Door options so that Open and Close can be specified separately

#### v1.0.0 [2022-01-23]
- Initial release
